/*
    Aurorae Designer: an application to design Aurorae themes.
    Copyright (C) 2010  Martin Gräßlin <kde@martin-graesslin.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#ifndef AURORAEDESIGNER_H
#define AURORAEDESIGNER_H


#include <kxmlguiwindow.h>

#include "ui_prefs_base.h"
#include <KTemporaryFile>

class ConfigModel;
namespace Aurorae {
    class AuroraeTheme;
}
class AuroraeDesignerView;
class QPrinter;
class KAction;
class KToggleAction;
class KUrl;

/**
 * This class serves as the main window for AuroraeDesigner.  It handles the
 * menus, toolbars, and status bars.
 *
 * @short Main window class
 * @author %{AUTHOR} <%{EMAIL}>
 * @version %{VERSION}
 */
class AuroraeDesigner : public KXmlGuiWindow
{
    Q_OBJECT
public:
    /**
     * Default Constructor
     */
    AuroraeDesigner();

    /**
     * Default Destructor
     */
    virtual ~AuroraeDesigner();

private slots:
    void fileNew();
    void fileOpen();
    void optionsPreferences();
    void configChanged();
    void save();
    void reload();
    void compositingChanged(bool active);
    void complianceDialog();
    void borderSizesTriggered(QAction* action);
    void buttonSizesTriggered(QAction* action);
    void ghns();
    void enableRemoveTab(bool enable);

private:
    void setupActions();
    void setupButtonAction(int type, bool left);
    void loadTheme(const QString &name);
    void enableButtonActions();
    KAction *addBorderAction(const QString &title, const QString &name, int border);

private:
    Ui::prefs_base ui_prefs_base ;
    Aurorae::AuroraeTheme *m_theme;
    AuroraeDesignerView *m_view;

    QPrinter   *m_printer;
    KToggleAction *m_toolbarAction;
    KToggleAction *m_statusbarAction;
    ConfigModel *m_model;
    KTemporaryFile m_tempFile;
    KTemporaryFile m_tempDesktopFile;
    KSharedConfigPtr m_config;
    KSharedConfigPtr m_desktopFile;
};

#endif // _AURORAEDESIGNER_H_
