/*
    Aurorae Designer: an application to design Aurorae themes.
    Copyright (C) 2010  Martin Gräßlin <kde@martin-graesslin.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/
#ifndef AURORAEDESIGNERVIEW_H
#define AURORAEDESIGNERVIEW_H

#include <QtGui/QGraphicsView>

class QGraphicsLinearLayout;
namespace Aurorae {
    class AuroraeTheme;
}

/**
 * This is the main view class for AuroraeDesigner.  Most of the non-menu,
 * non-toolbar, and non-statusbar (e.g., non frame) GUI code should go
 * here.
 *
 * @short Main view
 * @author %{AUTHOR} <%{EMAIL}>
 * @version %{VERSION}
 */

class AuroraeDesignerView : public QGraphicsView
{
    Q_OBJECT
public:
    /**
     * Default constructor
     */
    AuroraeDesignerView(QWidget *parent, Aurorae::AuroraeTheme *theme);
    virtual void drawForeground(QPainter* painter, const QRectF& rect);
    virtual void resizeEvent(QResizeEvent* event);

    /**
     * Destructor
     */
    virtual ~AuroraeDesignerView();

public Q_SLOTS:
    void slotShowGrid(bool enabled);
    void slotActiveChanged(bool active);
    void slotMaximizeChanged(bool maximized);
    void slotMaximizeChanged(Qt::MouseButtons buttons);
    void slotAllDesktopsToggled();
    void slotKeepAboveToggled();
    void slotKeepBelowToggled();
    void slotShadeToggled();
    void settingsChanged();
    void themeChanged();
    void leftButtonTriggered(bool active);
    void rightButtonTriggered(bool active);
    void addTab();
    void removeTab();
    void tabClicked(QGraphicsSceneMouseEvent* event, int index);

Q_SIGNALS:
    void maximized(bool maximized);
    void enableRemoveTab(bool enable);

private Q_SLOTS:
    void themeChangedTimeout();

private:
    void drawGrid(QPainter *painter);
    Aurorae::AuroraeTheme *m_theme;
    bool m_showGrid;
};

#endif // AuroraeDesignerVIEW_H
