/*
    Aurorae Designer: an application to design Aurorae themes.
    Copyright (C) 2010  Martin Gräßlin <kde@martin-graesslin.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/

#ifndef COMPLIANCEMODEL_H
#define COMPLIANCEMODEL_H

#include <QAbstractItemModel>

namespace Aurorae {
class AuroraeTheme;
}

class ComplianceModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    enum TopLevels {
        Decoration = 0,
        MinimizeButton,
        MaximizeButton,
        RestoreButton,
        CloseButton,
        AllDesktopsButton,
        KeepAboveButton,
        KeepBelowButton,
        ShadeButton,
        HelpButton,
        MenuButton,
        TopLevelCount
    };
    enum DecorationLevel {
        InactiveElement = 0,
        OpaqueElement,
        OpaueInactiveElement,
        MaximizedElement,
        MaximizedInactiveElement,
        MaximizedOpaqueElement,
        MaximizedOpaqueInactiveElement,
        InnerBorderElement,
        InnerBorderInactiveElement,
        TabActiveFocused,
        TabActiveUnfocused,
        TabInactiveFocused,
        TabInactiveUnfocused,
        DecorationCount
    };
    enum ButtonLevel {
        InactiveButtonElement = 0,
        HoverElement,
        HoverInactiveElement,
        PressedElement,
        PressedInactiveElement,
        DeactivatedElement,
        DeactivatedInactiveElement,
        ButtonCount
    };
    ComplianceModel(Aurorae::AuroraeTheme *theme, QObject* parent = 0);
    virtual QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;
    virtual int columnCount(const QModelIndex& parent = QModelIndex()) const;
    virtual int rowCount(const QModelIndex& parent = QModelIndex()) const;
    virtual QModelIndex parent(const QModelIndex& child) const;
    virtual QModelIndex index(int row, int column, const QModelIndex& parent = QModelIndex()) const;
    virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;

private:
    QModelIndex createButtonIndex(int row, int column, const QModelIndex &parent) const;
    int mapIdToAuroraeButton(qint64 id) const;
    QString mapIdToButtonElement(int id) const;
    QString mapIdToButtonDescription(int id) const;
    QString mapIdToDecorationElement(int id) const;
    QString mapIdToDecorationDescription(int id) const;
    QString mapIdToTopLevelDescription(int id) const;
    Aurorae::AuroraeTheme *m_theme;
};

#endif // COMPLIANCEMODEL_H
