/*
    Aurorae Designer: an application to design Aurorae themes.
    Copyright (C) 2010  Martin Gräßlin <kde@martin-graesslin.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/

#include "configmodel.h"

#include <KDE/KLocale>
#include <KDebug>
#include <kdecoration.h>

static const QString GENERALNAMES[] = {
    I18N_NOOP("Active Text Color"),
    I18N_NOOP("Inactive Text Color"),
    I18N_NOOP("Active, Focused Tab Text Color"),
    I18N_NOOP("Active, Unfocused Tab Text Color"),
    I18N_NOOP("Inactive, Focused Tab Text Color"),
    I18N_NOOP("Inactive, Unfocused Tab Text Color"),
    I18N_NOOP("Use Text Shadow"),
    I18N_NOOP("Active Text Shadow Color"),
    I18N_NOOP("Inactive Text Shadow Color"),
    I18N_NOOP("Title Shadow Offset (X)"),
    I18N_NOOP("Title Shadow Offset (Y)"),
    I18N_NOOP("Use Halo on Active Title"),
    I18N_NOOP("Use Halo on Inactive Title"),
    I18N_NOOP("Horizontal Title Alignment"),
    I18N_NOOP("Vertical Title Alignment"),
    I18N_NOOP("Animation Duration"),
    I18N_NOOP("Left Button Order"),
    I18N_NOOP("Right Button Order"),
    I18N_NOOP("Use Shadow"),
    I18N_NOOP("Decoration Position")
};

static const QString GENERALCONFIG[] = {
    "ActiveTextColor",
    "InactiveTextColor",
    "ActiveFocusedTabColor",
    "ActiveUnfocusedTabColor",
    "InactiveFocusedTabColor",
    "InactiveUnfocusedTabColor",
    "UseTextShadow",
    "ActiveTextShadowColor",
    "InactiveTextShadowColor",
    "TextShadowOffsetX",
    "TextShadowOffsetY",
    "HaloActive",
    "HaloInactive",
    "TitleAlignment",
    "TitleVerticalAlignment",
    "Animation",
    "LeftButtons",
    "RightButtons",
    "Shadow",
    "DecorationPosition"
};

static const QVariant GENERALDEFAULTS[] = {
    QColor(Qt::black),
    QColor(Qt::black),
    QColor(Qt::black),
    QColor(Qt::black),
    QColor(Qt::black),
    QColor(Qt::black),
    false,
    QColor(Qt::white),
    QColor(Qt::white),
    0,
    0,
    false,
    false,
    "Left",
    "Center",
    0,
    KDecorationOptions::defaultTitleButtonsLeft(),
    KDecorationOptions::defaultTitleButtonsRight(),
    true,
    0
};

static const ConfigModel::DataTypes GENERALTYPES[] = {
    ConfigModel::ColorType,
    ConfigModel::ColorType,
    ConfigModel::ColorType,
    ConfigModel::ColorType,
    ConfigModel::ColorType,
    ConfigModel::ColorType,
    ConfigModel::BoolType,
    ConfigModel::ColorType,
    ConfigModel::ColorType,
    ConfigModel::IntType,
    ConfigModel::IntType,
    ConfigModel::BoolType,
    ConfigModel::BoolType,
    ConfigModel::StringType,
    ConfigModel::StringType,
    ConfigModel::IntType,
    ConfigModel::StringType,
    ConfigModel::StringType,
    ConfigModel::BoolType,
    ConfigModel::IntType
};

static const QString LAYOUTNAMES[] = {
    I18N_NOOP("Left Border"),
    I18N_NOOP("Right Border"),
    I18N_NOOP("Bottom Border"),
    I18N_NOOP("Top Border"),
    I18N_NOOP("Title Edge Top"),
    I18N_NOOP("Title Edge Bottom"),
    I18N_NOOP("Title Edge Left"),
    I18N_NOOP("Title Edge Right"),
    I18N_NOOP("Title Edge Top (Maximized)"),
    I18N_NOOP("Title Edge Bottom (Maximized)"),
    I18N_NOOP("Title Edge Left (Maximized)"),
    I18N_NOOP("Title Edge Right (Maximized)"),
    I18N_NOOP("Left Title Border"),
    I18N_NOOP("Right Title Border"),
    I18N_NOOP("Title Height"),
    I18N_NOOP("Button Width"),
    I18N_NOOP("Minimize Button Width"),
    I18N_NOOP("Maximize Button Width"),
    I18N_NOOP("Close Button Width"),
    I18N_NOOP("All-Desktops Button Width"),
    I18N_NOOP("Keep-Above Button Width"),
    I18N_NOOP("Keep-Below Button Width"),
    I18N_NOOP("Shade Button Width"),
    I18N_NOOP("Help Button Width"),
    I18N_NOOP("Menu Button Width"),
    I18N_NOOP("Button Height"),
    I18N_NOOP("Button Spacing"),
    I18N_NOOP("Button Margin Top"),
    I18N_NOOP("Explicit Button Spacer"),
    I18N_NOOP("Left Padding"),
    I18N_NOOP("Right Padding"),
    I18N_NOOP("Top Padding"),
    I18N_NOOP("Bottom Padding")
};

static const QString LAYOUTCONFIG[] = {
    "BorderLeft",
    "BorderRight",
    "BorderBottom",
    "BorderTop",
    "TitleEdgeTop",
    "TitleEdgeBottom",
    "TitleEdgeLeft",
    "TitleEdgeRight",
    "TitleEdgeTopMaximized",
    "TitleEdgeBottomMaximized",
    "TitleEdgeLeftMaximized",
    "TitleEdgeRightMaximized",
    "TitleBorderLeft",
    "TitleBorderRight",
    "TitleHeight",
    "ButtonWidth",
    "ButtonWidthMinimize",
    "ButtonWidthMaximizeRestore",
    "ButtonWidthClose",
    "ButtonWidthAlldesktops",
    "ButtonWidthKeepabove",
    "ButtonWidthKeepbelow",
    "ButtonWidthShade",
    "ButtonWidthHelp",
    "ButtonWidthMenu",
    "ButtonHeight",
    "ButtonSpacing",
    "ButtonMarginTop",
    "ExplicitButtonSpacer",
    "PaddingLeft",
    "PaddingRight",
    "PaddingTop",
    "PaddingBottom"
};

static const int LAYOUTDEFAULTS[] = {
    5, // border left
    5, // border right
    5, // border bottom
    0, // border top
    5, // title edge top
    5, // title edge bottom
    5, // title edge left
    5, // title edge right
    0, 0, 0, 0, // title edges maximized
    5, // title border left
    5, // title border right
    20, // title height
    20, // button width
    -1, -1, -1, -1, -1, -1, -1, -1, -1, // dependend button width
    20, // button height
    5, // button spacing
    0, // button margin top
    10, // explicit spacer
    0, 0, 0, 0 // padding
};

static const QString METADATANAMES[] = {
    I18N_NOOP("Name"),
    I18N_NOOP("Comment"),
    I18N_NOOP("Author"),
    I18N_NOOP("Email"),
    I18N_NOOP("Website"),
    I18N_NOOP("Version"),
    I18N_NOOP("License")
};

static const QString METADATACONFIG[] = {
    "Name",
    "Comment",
    "X-KDE-PluginInfo-Author",
    "X-KDE-PluginInfo-Email",
    "X-KDE-PluginInfo-Website",
    "X-KDE-PluginInfo-Version",
    "X-KDE-PluginInfo-License"
};

ConfigModel::ConfigModel(QObject* parent)
    : QAbstractItemModel(parent)
    , m_valid(false)
{
}

ConfigModel::~ConfigModel()
{
}

QVariant ConfigModel::data(const QModelIndex& index, int role) const
{
    if (!index.isValid()) {
        return QVariant();
    }
    if (!m_valid) {
        return QVariant();
    }
    if (role == Qt::DisplayRole || role == Qt::EditRole) {
        return displayData(index);
    }
    if (role == Qt::CheckStateRole) {
        return checkStateData(index);
    }
    return QVariant();
}

QVariant ConfigModel::displayData(const QModelIndex& index) const
{
    if (!index.parent().isValid()) {
        // top level entries
        switch (index.row()) {
        case General:
            return i18n("General");
        case Layout:
            return i18n("Layout");
        case MetaData:
            return i18n("Meta Data");
        default:
            return QVariant();
        }
    } else if (index.parent().internalId() == General) {
        if (index.column() == 0) {
            if (index.row() < GeneralOptionsCount) {
                return GENERALNAMES[index.row()];
            } else {
                return QVariant();
            }
        } else {
            QModelIndex sibling = index.sibling(index.row(), 0);
            if (sibling.isValid() && isCheckable(sibling)) {
                return QVariant();
            }
            return m_general.readEntry(GENERALCONFIG[index.row()], GENERALDEFAULTS[index.row()]);
        }
    } else if (index.parent().internalId() == Layout) {
        if (index.column() == 0) {
            if (index.row() < LayoutOptionsCount) {
                return LAYOUTNAMES[index.row()];
            } else {
                return QVariant();
            }
        } else {
            int defaultValue = LAYOUTDEFAULTS[index.row()];
            if (defaultValue == -1) {
                defaultValue = m_layout.readEntry(LAYOUTCONFIG[ButtonWidth],
                                                  LAYOUTDEFAULTS[ButtonWidth]);
            }
            return m_layout.readEntry(LAYOUTCONFIG[index.row()], defaultValue);
        }
    } else if (index.parent().internalId() == MetaData) {
        if (index.column() == 0) {
            if (index.row() < MetaDataCount) {
                return METADATANAMES[index.row()];
            } else {
                return QVariant();
            }
        } else {
            return m_desktop.readEntry(METADATACONFIG[index.row()], QString());
        }
    } else {
        return QVariant();
    }
}

QVariant ConfigModel::checkStateData(const QModelIndex& index) const
{
    if (!index.parent().isValid()) {
        return QVariant();
    }
    if (index.parent().internalId() == General && index.column() == 0) {
        if (!m_general.isValid()) {
            return QVariant();
        }
        if (isCheckable(index)) {
            bool isChecked = m_general.readEntry<bool>(GENERALCONFIG[index.row()],
                                                       GENERALDEFAULTS[index.row()].toBool());
            return isChecked ? Qt::Checked : Qt::Unchecked;
        }
    }
    return QVariant();
}

int ConfigModel::columnCount(const QModelIndex& parent) const
{
    Q_UNUSED(parent)
    if (!m_valid) {
        return 0;
    }
    return 2;
}

int ConfigModel::rowCount(const QModelIndex& parent) const
{
    if (!m_valid) {
        return 0;
    }
    if (!parent.isValid()) {
        return GroupsCount;
    } else if (parent.internalId() == General) {
        return GeneralOptionsCount;
    } else if (parent.internalId() == Layout) {
        return LayoutOptionsCount;
    } else if (parent.internalId() == MetaData) {
        return MetaDataCount;
    } else {
        return 0;
    }
}

QModelIndex ConfigModel::parent(const QModelIndex& child) const
{
    if (!m_valid) {
        return QModelIndex();
    }
    if (child.internalId() < GroupsCount) {
        // parent object
        return QModelIndex();
    } else if (child.internalId() < 1000) {
        return createIndex(General, 0, General);
    } else if (child.internalId() < 10000){
        return createIndex(Layout, 0, Layout);
    } else {
        return createIndex(MetaData, 0, MetaData);
    }
}

QModelIndex ConfigModel::index(int row, int column, const QModelIndex& parent) const
{
    if (!m_valid) {
        return QModelIndex();
    }
    if (!parent.isValid()) {
        // top level
        if (row >= GroupsCount || column > 0) {
            // sanity check
            return QModelIndex();
        }
        return createIndex(row, column, row);
    } else if (parent.internalId() < GroupsCount) {
        if (parent.row() == General && row < GeneralOptionsCount) {
            return createIndex(row, column, 100+row);
        } else if (parent.row() == Layout && row < LayoutOptionsCount) {
            return createIndex(row, column, 1000+row);
        } else if (parent.row() == MetaData && row < MetaDataCount) {
            return createIndex(row, column, 10000+row);
        }
    }
    return QModelIndex();
}

bool ConfigModel::setData(const QModelIndex& index, const QVariant& value, int role)
{
    if (!index.isValid() || !index.parent().isValid()) {
        return QAbstractItemModel::setData(index, value, role);
    }
    if (role == Qt::CheckStateRole) {
        if (index.parent().internalId() == General && index.column() == 0) {
            if (isCheckable(index)) {
                m_general.writeEntry(GENERALCONFIG[index.row()], value.toBool());
                m_general.sync();
                emit dataChanged(index, index);
                return true;
            }
        }
    }
    if (role == Qt::EditRole) {
        if (index.parent().internalId() == General && index.column() == 1) {
            switch (GENERALTYPES[index.row()]) {
            case IntType:
                m_general.writeEntry(GENERALCONFIG[index.row()], value.toInt());
                break;
            case ColorType:
                m_general.writeEntry(GENERALCONFIG[index.row()], value.value<QColor>());
                break;
            case StringType:
                m_general.writeEntry(GENERALCONFIG[index.row()], value.toString());
                break;
            default:
                return QAbstractItemModel::setData(index, value, role);
            }
            m_general.sync();
            emit dataChanged(index, index);
            return true;
        }
        if (index.parent().internalId() == Layout && index.column() == 1) {
            m_layout.writeEntry(LAYOUTCONFIG[index.row()], value.toInt());
            m_layout.sync();
            emit dataChanged(index, index);
            return true;
        }
        if (index.parent().internalId() == MetaData && index.column() == 1) {
            m_desktop.writeEntry(METADATACONFIG[index.row()], value.toString());
            m_desktop.sync();
            emit dataChanged(index, index);
            return true;
        }
    }
    return QAbstractItemModel::setData(index, value, role);
}

Qt::ItemFlags ConfigModel::flags(const QModelIndex& index) const
{
    if (!index.isValid() || !index.parent().isValid()) {
        return QAbstractItemModel::flags(index);
    }
    Qt::ItemFlags ret = Qt::ItemIsSelectable;
    if (m_valid) {
        ret |= Qt::ItemIsEnabled;
    }
    if (isCheckable(index)) {
        ret |= Qt::ItemIsUserCheckable | Qt::ItemIsEditable;
    }
    if ((index.parent().internalId() == General || index.parent().internalId() == Layout ||
        index.parent().internalId() == MetaData)
        && index.column() == 1) {
        QModelIndex sibling = index.sibling(index.row(), 0);
        if (sibling.isValid() && isCheckable(sibling)) {
            // do not want to edit the item next to checkable items
            return ret;
        }
        ret |= Qt::ItemIsEditable;
    }
    return ret;
}

bool ConfigModel::isCheckable(const QModelIndex& index) const
{
    if (!index.parent().isValid()) {
        return false;
    }
    if (index.parent().internalId() == General && index.column() == 0) {
        switch (index.row()) {
            case UseTextShadow:
            case HaloActive:
            case HaloInacitve:
            case UseShadow:
                return true;
            default:
                break; // nothing
        }
    }
    return false;
}

void ConfigModel::setConfigGroups(KSharedConfigPtr config, KSharedConfigPtr desktopFile)
{
    beginResetModel();
    m_general = config->group("General");
    m_layout = config->group("Layout");
    m_desktop = desktopFile->group("Desktop Entry");
    m_valid = true;
    endResetModel();
}

QVariant ConfigModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
        switch (section) {
        case 0:
            return i18n("Option");
        case 1:
            return i18n("Value");
        default:
            break; // fallback to default
        }
    }
    return QAbstractItemModel::headerData(section, orientation, role);
}
