/*
    Aurorae Designer: an application to design Aurorae themes.
    Copyright (C) 2010  Martin Gräßlin <kde@martin-graesslin.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

*/

#ifndef CONFIGMODEL_H
#define CONFIGMODEL_H

#include <QtCore/QAbstractItemModel>
#include <KConfigGroup>

class ConfigModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    enum Groups {
        General = 0,
        Layout = 1,
        MetaData = 2,
        GroupsCount = 3
    };
    enum GeneralOptions {
        ActiveTextColor = 0,
        InactiveTextColor,
        ActiveFocusedTabColor,
        ActiveUnfocusedTabColor,
        InactiveFocusedTabColor,
        InactiveUnfocusedTabColor,
        UseTextShadow,
        ActiveShadowColor,
        InactiveShadowColor,
        ShadowOffsetX,
        ShadowOffsetY,
        HaloActive,
        HaloInacitve,
        TitleAlignment,
        TitleVerticalAlignment,
        Animation,
        LeftButtons,
        RightButtons,
        UseShadow,
        DecorationPosition,
        GeneralOptionsCount
    };
    enum LayoutOptions {
        BorderLeft = 0,
        BorderRight,
        BorderBottom,
        BorderTop,
        TitleEdgeTop,
        TitleEdgeBottom,
        TitleEdgeLeft,
        TitleEdgeRight,
        TitleEdgeTopMaximized,
        TitleEdgeBottomMaximized,
        TitleEdgeLeftMaximized,
        TitleEdgeRightMaximized,
        TitleBorderLeft,
        TitleBorderRight,
        TitleHeight,
        ButtonWidth,
        ButtonWidthMinimize,
        ButtonWidthMaximizeRestore,
        ButtonWidthClose,
        ButtonWidthAlldesktops,
        ButtonWidthKeepabove,
        ButtonWidthKeepbelow,
        ButtonWidthShade,
        ButtonWidthHelp,
        ButtonWidthMenu,
        ButtonHeight,
        ButtonSpacing,
        ButtonMarginTop,
        ExplicitButtonSpacer,
        PaddingLeft,
        PaddingRight,
        PaddingTop,
        PaddingBottom,
        LayoutOptionsCount
    };
    enum MetaDataOptions {
        NameData = 0,
        CommentData,
        AuthorData,
        EmailData,
        WebsiteData,
        VersionData,
        LicenseData,
        MetaDataCount
    };
    enum DataTypes {
        IntType = 0,
        BoolType = 2,
        StringType = 3,
        ColorType = 4
    };
    ConfigModel(QObject* parent = 0);
    virtual ~ConfigModel();
    virtual QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const;
    virtual int columnCount(const QModelIndex& parent = QModelIndex()) const;
    virtual int rowCount(const QModelIndex& parent = QModelIndex()) const;
    virtual QModelIndex parent(const QModelIndex& child) const;
    virtual QModelIndex index(int row, int column, const QModelIndex& parent = QModelIndex()) const;
    virtual bool setData(const QModelIndex& index, const QVariant& value, int role = Qt::EditRole);
    virtual Qt::ItemFlags flags(const QModelIndex& index) const;
    virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;

    void setConfigGroups(KSharedConfigPtr config, KSharedConfigPtr desktopFile);

private:
    QVariant displayData(const QModelIndex &index) const;
    QVariant checkStateData(const QModelIndex &index) const;
    bool isCheckable(const QModelIndex &index) const;

    bool m_valid;
    KConfigGroup m_general;
    KConfigGroup m_layout;
    KConfigGroup m_desktop;
};

#endif // CONFIGMODEL_H
